#include "GLSLModel.h"


void GLSLModel::LoadShaderFile(GLuint id, const char *argFileName)
{
	// t@C̒gSēǂݍ
	std::ifstream inputFile1(argFileName);
	std::istreambuf_iterator<char> vdataBegin(inputFile1);
	std::istreambuf_iterator<char> vdataEnd;
	std::string fileData1(vdataBegin,vdataEnd);
	const char *pFile = fileData1.c_str();

	// VF[_RpC
	glShaderSource(id, 1, &pFile, NULL);
	glCompileShader(id);
	// G[`FbN͕ʊ֐ɂ

	return;
}

bool GLSLModel::PrintShaderInfoLog(GLuint shader)
{
	GLsizei bufSize;

	/* VF[_̃RpC̃O̒擾 */
	glGetShaderiv(shader, GL_INFO_LOG_LENGTH , &bufSize);

	if(bufSize > 1) {
		GLchar *infoLog = (GLchar *)malloc(bufSize);

		if(infoLog != NULL) {
			GLsizei length;

			/* VF[_̃RpC̃O̓e擾 */
			glGetShaderInfoLog(shader, bufSize, &length, infoLog);
			fk_Printf("InfoLog:\n%s\n\n", infoLog);
			free(infoLog);
		} else {
			fk_Printf("Could not allocate InfoLog buffer.\n");
		}

		return false;
	}

	return true;
}

GLSLModel::GLSLModel(void) : fk_Model(),
	vpID(0), fpID(0), shID(0), useTex(false), useSatBle(false),
	ref_image(NULL), texID(0)
{
	return;
}

GLSLModel::~GLSLModel()
{
	glDeleteShader(vpID);
	glDeleteShader(fpID);
	glDeleteProgram(shID);
}

void GLSLModel::initShader(const char  *argVPName, const char *argFPName)
{
	vpID = glCreateShader(GL_VERTEX_SHADER);
	LoadShaderFile(vpID, argVPName);
	PrintShaderInfoLog(vpID);

	fpID = glCreateShader(GL_FRAGMENT_SHADER);
	LoadShaderFile(fpID, argFPName);
	PrintShaderInfoLog(fpID);

	shID = glCreateProgram();
	glAttachShader(shID, vpID);
	glAttachShader(shID, fpID);

	GLint	linked = 0;
	glLinkProgram(shID);
	glGetProgramiv(shID, GL_LINK_STATUS, &linked);
	if(linked == GL_FALSE) {
		fk_Printf("Shader Link Err.\n");
	}

	return;
}

void GLSLModel::setTexUse(bool argMode)
{
	useTex = argMode;
	if(useTex) {
		glUseProgram(shID);
		glUniform1i(glGetUniformLocation(shID, "tex0"), 0);
		glUseProgram(0);
	}
}

void GLSLModel::setSatUse(bool argMode)
{
	useSatBle = argMode;
}

void GLSLModel::preShader(void)
{
	if(useSatBle) glBlendFunc(GL_ONE, GL_ONE);
	glUseProgram(shID);
	if(ref_image != NULL) {
		// VF[_p̒ǉeNX`̓jbg1ȍ~g
		glActiveTexture(GL_TEXTURE1);
		// ǂݍ񂾃eNX`KpAjbgԍn
		glBindTexture(GL_TEXTURE_2D, texID);
		glUniform1i(glGetUniformLocation(shID, "toonTex"), 1);
		// ǉeNX`𑀍삵Ajbg0ɖ߂Ă
		glActiveTexture(GL_TEXTURE0);
	}

	return;
}	

void GLSLModel::postShader(void)
{
	glUseProgram(0);
	if(useSatBle) glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	return;
}

void GLSLModel::setImage(fk_Image *argImage)
{
	ref_image = argImage;
	if(ref_image == NULL) return;

	const fk_ImType		*imageBuf = argImage->getBufPointer();
	const fk_Dimension	*bufSize = argImage->getBufferSize();

	// eNX`ID𐶐
	if(texID != 0) glDeleteTextures(1, &texID);
	glGenTextures(1, &texID);
	// IDɑ΂ăeNX`̐ݒs
	glBindTexture(GL_TEXTURE_2D, texID);
	// eNX`̌JԂp^[ǂ邩ݒ
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);			// eNX`̍E̓s[g
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);	// ㉺̓s[gȂ悤ɂ
	// eNX``ԂYɕ`AWM点邩ݒ肷
	// {͐`ԂYɕ`[hOK
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

	// fk_Imageɓǂݍ񂾉摜eNX`Ƃē]
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, bufSize->w, bufSize->h,
				0, GL_RGBA, GL_UNSIGNED_BYTE, imageBuf);

	// eNX`삪ς񂾂ID0ɐ؂ւĂ
	glBindTexture(GL_TEXTURE_2D, 0);

	return;
}
